# coding: utf-8

"""
Gluish - some glue code and utils for luigi.

GLUISH_DATA: where common task place their artefacts
"""

import requests

requests.adapters.DEFAULT_RETRIES = 3
__version__ = '0.1.39'

# environment variable name
GLUISH_DATA = 'GLUISH_DATA'



#!/usr/bin/env python
# coding: utf-8

"""
Provides a basic benchmark decorator. Usage:

    @timed
    def run(self):
        print('Running...')

Just logs the output. Could be extended to send this information to some service
if available.
"""

from functools import wraps
from gluish.colors import green, yellow, red
from timeit import default_timer
import logging

logger = logging.getLogger('gluish')


class Timer(object):
    """ A timer as a context manager. """

    def __init__(self, green=10, yellow=60):
        """ Indicate runtimes with colors, green < 10s, yellow < 60s. """
        self.timer = default_timer
        # measures wall clock time, not CPU time!
        # On Unix systems, it corresponds to time.time
        # On Windows systems, it corresponds to time.clock

        self.green = green
        self.yellow = yellow

    def __enter__(self):
        self.start = self.timer() # measure start time
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.end = self.timer() # measure end time
        self.elapsed_s = self.end - self.start # elapsed time, in seconds
        self.elapsed_ms = self.elapsed_s * 1000  # elapsed time, in milliseconds


def timed(method):
    """ A @timed decorator. """
    @wraps(method)
    def timed(*args, **kwargs):
        """ Benchmark decorator. """
        with Timer() as timer:
            result = method(*args, **kwargs)
        klass = args[0].__class__.__name__
        fun = method.__name__

        msg = '[%s.%s] %0.5f' % (klass, fun, timer.elapsed_s)
        if timer.elapsed_s <= timer.green:
            logger.debug(green(msg))
        elif timer.elapsed_s <= timer.yellow:
            logger.debug(yellow(msg))
        else:
            logger.debug(red(msg))
        return result
    return timed



# coding: utf-8

"""
Color helpers.
"""

from colorama import Fore, Back, Style


def dim(text):
    """ Dim text. """
    return Back.WHITE + Fore.BLACK + text + Fore.RESET + Back.RESET

def green(text):
    """ Green text. """
    return Fore.GREEN + text + Fore.RESET

def red(text):
    """ Red text. """
    return Fore.RED + text + Fore.RESET

def yellow(text):
    """ Yellow text. """
    return Fore.YELLOW + text + Fore.RESET

def cyan(text):
    """ Cyan text. """
    return Fore.CYAN + text + Fore.RESET

def magenta(text):
    """ Magenta text. """
    return Fore.MAGENTA + text + Fore.RESET



# coding: utf-8

"""
Configuration handling, adapted from luigi.
"""

from ConfigParser import ConfigParser, NoOptionError, NoSectionError
import datetime


class BaseConfigParser(ConfigParser):
    """ A ConfigParser, that can be queried for typed configuration entries. """
    NO_DEFAULT = None
    _instance = None
    _config_paths = []

    @classmethod
    def add_config_path(cls, path):
        """ Append a path to read from. """
        cls._config_paths.append(path)
        cls.instance().reload()

    @classmethod
    def instance(cls, *args, **kwargs):
        """ Singleton getter """
        if cls._instance is None:
            cls._instance = cls(*args, **kwargs)
            _ = cls._instance.reload()

        return cls._instance

    def reload(self):
        """ Reload the configuration. """
        return self._instance.read(self._config_paths)

    def _get_with_default(self, method, section, option, default,
                          expected_type=None):
        """
        Gets the value of the section/option using method. Returns default
        if value is not found. Raises an exception if the default value
        is not None and doesn't match the expected_type.
        """
        try:
            return method(self, section, option)
        except (NoOptionError, NoSectionError):
            if default is BaseConfigParser.NO_DEFAULT:
                raise
            if (expected_type is not None and default is not None and
                not isinstance(default, expected_type)):
                raise
            return default

    def get(self, section, option, default=NO_DEFAULT):
        """ Default. """
        return self._get_with_default(ConfigParser.get, section, option,
                                      default)

    def getboolean(self, section, option, default=NO_DEFAULT):
        """ Return a boolean. """
        return self._get_with_default(ConfigParser.getboolean, section, option,
                                      default, bool)

    def getint(self, section, option, default=NO_DEFAULT):
        """ Return an int. """
        return self._get_with_default(ConfigParser.getint, section, option,
                                      default, int)

    def getfloat(self, section, option, default=NO_DEFAULT):
        """ Return a float. """
        return self._get_with_default(ConfigParser.getfloat, section, option,
                                      default, float)

    def getdate(self, section, option, default=NO_DEFAULT, fmt='%Y-%m-%d'):
        """ Return a datetime.date instance. """
        value = self.get(section, option, default=default)
        return datetime.datetime.strptime(value, fmt).date()

    def getdatetime(self, section, option, default=NO_DEFAULT,
                    fmt='%Y-%m-%d'):
        """ Return a datetime. """
        value = self.get(section, option, default=default)
        return datetime.datetime.strptime(value, fmt)


def get_config():
    """ Convenience method (for backwards compatibility) for accessing
        config singleton """
    return BaseConfigParser.instance()




# coding: utf-8

"""
OAI-PHM harvesting helper.
"""

from gluish.path import unlink
from gluish.utils import shellout, download, random_string
import BeautifulSoup
import logging
import os
import urllib

logger = logging.getLogger('gluish')


def oai_harvest(url=None, collection=None, begin=None, end=None,
                prefix='oai_dc', verb='ListRecords',
                max_retries=8, directory=None, ext='xml', download=download):
    """
    Harvest OAI for `url`. Will download all files into `directory`

    argument    OAI name
    --------------------
    begin       from
    collection  set
    end         until
    prefix      metadataPrefix
    verb        verb
    """
    if url is None:
        raise RuntimeError('A URL must be given.')
    if directory is None:
        raise RuntimeError('A directory must be given.')
    if not os.path.exists(directory):
        raise RuntimeError('Directory does not exist: %s' % directory)

    params = {'from': begin, 'until': end,
              'metadataPrefix': prefix, 'set': collection,
              'verb': verb}
    params = dict([(k, v) for k, v in params.iteritems() if v])

    # first request with all params
    full_url = '%s?%s' % (url, urllib.urlencode(params))
    path = os.path.join(directory, '%s.%s' % (random_string(length=16), ext))

    for retry in range(max_retries):
        try:
            download(url=full_url, filename=path, timeout=30)
            break
        except RuntimeError as err:
            logger.info('Retry %s on %s' % (retry, full_url))
            unlink(path)
    else:
        raise RuntimeError('Max retries (%s) exceeded: %s' % (
                           max_retries, full_url))

    # any subsequent request uses 'resumptiontoken'
    while True:
        with open(path) as handle:
            soup = BeautifulSoup.BeautifulStoneSoup(handle.read())
        token = soup.find('resumptiontoken')
        if token is None:
            break

        # subsequent requests are done with resumptiontoken only ...
        params = {'resumptionToken': token.text, 'verb': verb}
        full_url = '%s?%s' % (url, urllib.urlencode(params))
        path = os.path.join(directory, "%s.%s" % (
                            random_string(length=16), ext))

        retry = 0
        while True:
            if retry >= max_retries:
                raise RuntimeError("Max retries (%s) exceeded: %s" % (
                                   max_retries, full_url))
            try:
                download(url=full_url, filename=path)
                break
            except RuntimeError as err:
                retry += 1
                logger.info("Retry #%s on %s" % (retry, full_url))
                unlink(path)



#!/usr/bin/env python
# coding: utf-8

"""
Default task
============

A default task, that covers file system layout.

"""
# pylint: disable=F0401,E1101,E1103
from gluish import GLUISH_DATA
from gluish.parameter import ClosestDateParameter
from luigi.task import id_to_name_and_params
import datetime
import hashlib
import luigi
import os
import tempfile


def is_closest_date_parameter(task, param_name):
    """ Return the parameter class of param_name on task. """
    for name, obj in task.get_params():
        if name == param_name:
            return hasattr(obj, 'use_closest_date')
    return False


class BaseTask(luigi.Task):
    """
    A base task with a `path` method. BASE should be set to the root
    directory of all tasks. TAG is a shard for a group of related tasks.
    """
    BASE = os.environ.get(GLUISH_DATA, tempfile.gettempdir())
    TAG = 'default'

    def closest(self):
        """ Return the closest date for a given date.
        Defaults to the same date. """
        return self.date

    def path(self, filename=None, ext='tsv', digest=False):
        """
        Return the path for this class with a certain set of parameters.
        `ext` sets the extension of the file.
        If `hash` is true, the filename (w/o extenstion) will be hashed.
        """
        if self.TAG is NotImplemented or self.BASE is NotImplemented:
            raise RuntimeError('TAG and BASE must be set.')
        
        task_name, task_params = id_to_name_and_params(self.task_id)

        if filename is None:
            if 'date' in task_params:
                if is_closest_date_parameter(self, 'date'):
                    task_params['date'] = self.closest()

            parts = ('{k}-{v}'.format(k=k, v=v)
                     for k, v in task_params.iteritems())

            name = '-'.join(sorted(parts))
            if len(name) == 0:
                name = 'output'
            if digest:
                name = hashlib.sha1(name).hexdigest()
            if not ext:
                filename = '{fn}'.format(ext=ext, fn=name)
            else:
                filename = '{fn}.{ext}'.format(ext=ext, fn=name)

        return os.path.join(unicode(self.BASE), unicode(self.TAG), task_name,
                            filename)


class MockTask(BaseTask):
    """ A mock task object. Read fixture from path and that's it. """
    fixture = luigi.Parameter()

    def content(self):
        """ Return the content of the file in path. """
        with open(self.fixture) as handle:
            return handle.read()

    def run(self):
        """ Just copy the fixture, so we have some output. """
        luigi.File(path=self.fixture).copy(self.output().path)

    def output(self):
        """ Mock output. """
        return luigi.LocalTarget(path=self.path(digest=True))




# coding: utf-8

"""
Some glue around luigi.

Provides a base class, that autogenerates its output filenames based on some
tag, classname and parameters.

Additionally, provide some smaller utilities, like a TSV format, a benchmark
decorator and some task templates.
"""
from setuptools import setup


setup(name='gluish',
      version='0.1.39',
      description='Utils around Luigi.',
      url='https://github.com/miku/gluish',
      author='Martin Czygan',
      author_email='martin.czygan@gmail.com',
      packages=[
        'gluish',
      ],
      package_dir={'gluish': 'gluish'},
      install_requires=[
        # 'MySQL-python==1.2.5',
        'BeautifulSoup==3.2.1',
        'colorama==0.2.7',
        'elasticsearch==1.0.0',
        'luigi==1.0.16',
        'nose==1.3.1',
        'pyisbn==1.0.0',
        'python-dateutil==2.2',
        'requests>=2.2.1',
        'six==1.6.1',
        'sqlitebck==1.2.1',
        'urllib3==1.8',
        'wsgiref==0.1.2',
      ],
)
